#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# File name          : Apache-Tomcat-Scanner.py
# Author             : Podalirius (@podalirius_)
# Date created       : 24 Jul 2022


from apachetomcatscanner.__main__ import main

if __name__ == '__main__':
    main()

